import hashlib
import time

def double_sha256(input_string):
    """Perform double SHA-256 hashing."""
    first_hash = hashlib.sha256(input_string.encode()).digest()
    second_hash = hashlib.sha256(first_hash).hexdigest()
    return second_hash

def count_leading_zeros(hex_str):
    """Count the number of leading zeros in a hexadecimal hash."""
    return len(hex_str) - len(hex_str.lstrip('0'))

# Initialize tracking variables
best_candidate = None
best_hash = None
max_zeros = 0
start_time = time.time()

# Run the search loop for 100,000,000 numbers
for num in range(1000_000_000001, 10000_000_000001):
    input_string = "Bitcoin" + str(num)
    hash_result = double_sha256(input_string)
    zeros = count_leading_zeros(hash_result)

    # If the new hash has more leading zeros, update the best found
    if zeros > max_zeros:
        best_candidate = num
        best_hash = hash_result
        max_zeros = zeros

        # Print the best found so far
        print(f"New Best Found: Integer={best_candidate}, Hash={best_hash}, Leading Zeros={max_zeros}")

    # Print progress every 1M iterations
    if num % 1_000_000 == 0:
        elapsed_time = time.time() - start_time
        print(f"Checked {num:,} numbers in {elapsed_time:.2f} seconds...")

# Final result
print("\nFINAL BEST RESULT:")
print(f"Integer={best_candidate}, Hash={best_hash}, Leading Zeros={max_zeros}")

